// Copyright 2012 Concrete Software. All rights reserved.

uniform sampler2D texture;
// Direction of the main light in eye coordinates. Must be normalized.
uniform mediump vec3 pba_KeyLightPosition;
// Shininess amount. 128 is very shiny. 1 is not so shiny.
uniform mediump float pba_KeyShininess;
// values for "Rim" light on side to bring out shape
uniform mediump vec3 pba_RimLightPosition;
//uniform lowp float pba_RimShininess;
// color (and intensity) for the diffuse light
uniform mediump vec3 pba_diffuseColor;
// color (and intensity) for the ambient light
uniform mediump vec3 pba_ambientColor;
// color (and intensity) for the specular component of the light
uniform mediump vec3 pba_specularColor;

varying mediump vec3 var_Normal;
varying mediump vec3 var_TransformedVertex;
varying mediump vec2 var_TexCoord;

void main()
{
	mediump vec3 normalizedNormal = normalize(var_Normal);
	mediump float diffuseIntensity = max(dot(normalizedNormal, pba_KeyLightPosition), 0.0);
    diffuseIntensity += max(dot(normalizedNormal, pba_RimLightPosition), 0.0);
	
	mediump vec3 eyePosition = normalize(var_TransformedVertex);
	mediump vec3 halfLight = normalize(pba_KeyLightPosition - eyePosition);
	mediump float normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	mediump float specularComponent = pow(normalDotHalf, pba_KeyShininess);
	//halfLight = normalize(pba_RimLightPosition - eyePosition);
	//normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	//specularComponent += pow(normalDotHalf, pba_RimShininess);

	gl_FragColor = vec4(pba_ambientColor + pba_diffuseColor * diffuseIntensity, 1.0) * texture2D(texture, var_TexCoord) + vec4(specularComponent * pba_specularColor, 1.0);
    //gl_FragColor = vec4(1.0, 0.0, 1.0, 1.0);//;

	// * texture2D(texture, var_TexCoord);
}
