// Basic per-vertex lighting shader for PBA Challenge.
// Copyright 2012 Concrete Software. All rights reserved.

uniform mat4 cs_ModelViewProjectionMatrix;
// Model view matrix without projection multiplied in. Needed for specular lighting.
//uniform mat4 cs_ModelViewMatrix;
uniform mat3 cs_NormalMatrix;
// Direction of the light in eye coordinates. Must be normalized.
uniform vec3 pba_KeyLightPosition;
// Shininess amount. 128 is very shiny. 1 is not so shiny.
//uniform float pba_KeyShininess;
// values for "Rim" light on side to bring out shape
uniform vec3 pba_RimLightPosition;
// color (and intensity) for the diffuse light
uniform vec3 pba_diffuseColor;
// color (and intensity) for the ambient light
uniform vec3 pba_ambientColor;

attribute vec4 cs_Vertex;
attribute vec3 cs_Normal;
attribute vec2 cs_MultiTexCoord0;

varying lowp vec4 var_Color;
//varying lowp float var_SpecularComponent;
varying mediump vec2 var_TexCoord;

void main()
{
	vec3 eyeNormal = cs_NormalMatrix * cs_Normal / 127.0;
    
	float diffuseIntensity = max(dot(eyeNormal, pba_KeyLightPosition), 0.0);
    diffuseIntensity += max(dot(eyeNormal, pba_RimLightPosition), 0.0);
	
    //vec3 eyePosition = normalize((cs_ModelViewMatrix * cs_Vertex).xyz);
    //vec3 halfLight = normalize(pba_KeyLightPosition - eyePosition);
    //float normalDotHalf = max(dot(eyeNormal, halfLight), 0.);
    
	//var_SpecularComponent = pow(normalDotHalf, pba_KeyShininess);
	var_Color = vec4(pba_ambientColor + pba_diffuseColor * diffuseIntensity, 1);
	var_TexCoord = cs_MultiTexCoord0.xy;
	gl_Position = cs_ModelViewProjectionMatrix * cs_Vertex;
}
